#!/usr/bin/python3
# -*- coding: utf-8 -*-

__version__ = "0.1.0"
__author__ = "Choi, Chul <choi100472@naver.com>"
__copyright__ = ""


import re
import sys
import os.path
from gi.repository import Gio

try:
    from PIL import Image
except:
    import Image


#==================================================================== CONF ====

BASE_PATH = "/usr/share/dataman"

#Cover files list
COVER_FILES = ["cover.png", "cover.jpg", ".cover.png", ".cover.jpg",
        "Cover.png", "Cover.jpg", ".Cover.png", ".Cover.jpg",
        "folder.png", "folder.jpg", ".folder.png", ".folder.jpg",
        "Folder.png", "Folder.jpg", ".Folder.png", ".Folder.jpg"]

#Supported picture ext (ALWAY LAST 4 CHARS !!)
PICTURES_EXT = [".jpg", ".JPG", "jpeg", "JPEG",
        ".png", ".PNG", #Not interlaced
        ".gif", ".GIF",
        ".bmp", ".BMP", #Window ans OS/2 bitmap
        ".ico", ".ICO", #Windows icon format
        ".tga", ".TGA", #Truevision Targa format
        ".tif", ".TIF", "tiff", "TIFF", #Adobe Tagged Image File Format
        ".psd", ".PSD", #Adobe Photosop format (only version 2.5 and 3.0)
        ]

#==============================================================================


class Thumb(object):
    """ Makes thumbnails.

    Generate thumbnails for all kind of folders
    """
    def __init__(self, path):
        self.image = None
        self.thumb = None
        #
        preview = os.path.join(path, '.folder.png')
        try:
            self.image = Image.open(preview).convert("RGBA")
        except IOError:
            self.image = None
        #
        if not self.image:
            preview = os.path.join(path, '.folder', 'preview.png')
            try:
                self.image = Image.open(preview).convert("RGBA")
            except IOError:
                self.image = None

    def thumbnail(self):
        if self.image:
            # image = self.thumbnailize(self.image, size, crop=True)
            # if image.size[0] == size and image.size[1] == size:
            #     image.paste(fg, (0, 0), fg)
            #     self.thumb = image
            self.thumb = self.image

    def save(self, output_path, output_format='PNG'):
        """ Save the thumbnail in a file.

        Argument:
          * output_path -- the output path for the thumbnail
        Keyword argument:
          * format -- the format of the picture (PNG, JPEG,...)

        NOTE : The output format must be a PNG for a standard
               freedesktop thumbnail
        """
        if self.thumb is not None:
            self.thumb.save(output_path, output_format)
        else:
            print("E: [%s:Thumb.save_thumb] No thumbnail created" % __file__)



def match_path(path, path_list):
    """ Test if a folder is a sub-folder of another one in the list.

    Arguments
      * path -- path to check
      * path_list -- list of path
    """
    match = False
    #We add a slash at the end.
    if path[-1:] is not "/":
        path += "/"
    for entry in path_list:
        #We add a slash at the end.
        if entry[-1:] is not "/":
            entry += "/"
        if re.match(r"^" + entry + ".*", path):
            if path is not entry:
                match = True
                break
    return match


def gvfs_uri_to_path(uri):
    """Returns local file path from gvfs URI

    Arguments:
    uri -- the gvfs URI
    """
    if not re.match(r"^[a-zA-Z0-9_-]+://", uri):
        return uri
    gvfs = Gio.Vfs.get_default()
    return gvfs.get_file_for_uri(uri).get_path()


if __name__ == "__main__":
    #If we have 2 args
    if len(sys.argv) == 3:
        INPUT_FOLDER = gvfs_uri_to_path(sys.argv[1])
        OUTPUT_FILE = gvfs_uri_to_path(sys.argv[2])
    else:
        print("Dataman thumbnailer - %s" % __version__)
        # print(__copyright__)
        sys.exit(1)

    if os.path.isdir(INPUT_FOLDER):
        # TODO : 읽기/쓰기 권한 체크
        thumb = Thumb(INPUT_FOLDER)
        thumb.thumbnail()
        thumb.save(OUTPUT_FILE, "PNG")
    else:
        print("E: [%s:__main__] '%s' is not a directory" % (__file__, INPUT_FOLDER))
        sys.exit(2)


