#!/usr/bin/env python3
#-*- mode: python; coding: utf-8 -*-

import os
import io
try:
    from urllib import unquote
except ImportError:
    from urllib.parse import unquote
from xml.etree.ElementTree import ElementTree, parse

import gi
gi.require_version('Gtk', '3.0')
gi.require_version('GConf', '2.0')
gi.require_version('Nautilus', '3.0')
from gi.repository import Nautilus, Gtk, Gdk, GObject, GdkPixbuf, GConf
from PIL import Image


BASE_PATH  = "/usr/share/dataman"
IMAGE_PATH = os.path.join(BASE_PATH, "images")
ICON_PATH  = os.path.join(BASE_PATH, "icons")

PREVIEW_SIZE = 200

#--[ lib/xml.py ]---------------------------------------------------------------

XML_HEADER = "<?xml version='1.0' encoding='utf-8' ?>"

class XML():
    def __init__(self, xmlfile):
        tree = parse(xmlfile)
        self.tree = tree.getroot()

    def find(self, node, tag=None):
        if tag:
            try:
                value = self.tree.find(node + "/" + tag).text
            except:
                value = ""
        else:
            try:
                value = self.tree.find(node).text
            except:
                value = ""
        #
        if value:
            return value
        else:
            return ""


def xml_escape(s):
    xml_escape_table = {
        "&": "&amp;",
        # '"': "&quot;",
        # "'": "&apos;",
        ">": "&gt;",
        "<": "&lt;",
    }
    return "".join(xml_escape_table.get(c, c) for c in s)


def xml_unescape(s):
    s = s.replace("&lt;", "<")
    s = s.replace("&gt;", ">")
    # this has to be last:
    s = s.replace("&amp;", "&")
    return s


#--[ lib/image.py ]---------------------------------------------------------------
def image_to_pixbuf(image):
    f = io.BytesIO()
    image.save(f, 'png')
    data = f.getvalue()
    f.close()
    loader = GdkPixbuf.PixbufLoader()
    loader.write(data)
    loader.close()
    pixbuf = loader.get_pixbuf()
    return pixbuf



class ColumnExtension(GObject.GObject, Nautilus.LocationWidgetProvider):
    def __init__(self):
        pass
    
    def get_widget(self, uri, window):
        path = unquote(uri[7:])
        
        if not os.path.exists(os.path.join(path, ".folder")):
            return None

        infoxml = os.path.join(path, ".folder", "info.xml")
        preview = os.path.join(path, ".folder", "preview.png")
        
        if not os.path.isfile(infoxml) and  not os.path.isfile(preview):
            return None

        self.xml = XML(infoxml)
        self.xml_id = self.xml.find('form', 'id')
        if self.xml_id == "2004100700":
            # [개인정보] 사진+이름+전화+휴대폰+주소+사무실+전화+팩스+메모
            
            self.box = Gtk.HBox(spacing=5)
            self.box.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(1.0, 0.95, 0.95, 1.0))
            self.left = Gtk.VBox(spacing=2)
            self.right = Gtk.VBox(spacing=2)

            self.photo = Gtk.Image()
            self.photo.set_size_request(PREVIEW_SIZE, PREVIEW_SIZE)
            self.photo.set_alignment(0, 0)
            if os.path.isfile(preview):
                _image = Image.open(preview)
                _image.thumbnail((PREVIEW_SIZE, PREVIEW_SIZE), Image.ANTIALIAS)
                image = Image.new("RGBA", (PREVIEW_SIZE, PREVIEW_SIZE))
                image.paste(_image, (int((PREVIEW_SIZE - _image.size[0])/2), 0))
                self.photo.set_from_pixbuf(image_to_pixbuf(image))
            self.left.pack_start(self.photo, False, False, 10)

            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label(" "), False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "name.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 이 름 : "), False, False, 0)
            self.name = Gtk.Label(self.xml.find('form/data', 'name'))
            self.name.set_alignment(0, 0.5)
            self.name.set_selectable(True)
            self.name.set_size_request(220, -1)
            hbox.pack_start(self.name, False, False, 0)
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "office.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 직 장 : "), False, False, 0)
            self.office = Gtk.Label(self.xml.find("form/data/office", "name"))
            self.office.set_alignment(0, 0.5)
            self.office.set_selectable(True)
            self.office.set_size_request(220, -1)
            hbox.pack_start(self.office, False, False, 0)
            self.right.pack_start(hbox, False, False, 1)
            hbox = Gtk.HBox()
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "phone.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 전 화 : "), False, False, 0)
            self.phone = Gtk.Label(self.xml.find("form/data", "phone"))
            self.phone.set_alignment(0, 0.5)
            self.phone.set_selectable(True)
            self.phone.set_size_request(220, -1)
            hbox.pack_start(self.phone, False, False, 0)
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "phone.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 전 화 : "), False, False, 0)
            self.phone1 = Gtk.Label(self.xml.find(self.xml.find("form/data/office", "phone")))
            self.phone1.set_alignment(0, 0.5)
            self.phone1.set_selectable(True)
            self.phone1.set_size_request(220, -1)
            hbox.pack_start(self.phone1, False, False, 0)
            self.right.pack_start(hbox, False, False, 1)
            hbox = Gtk.HBox()
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "mobile.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 휴대폰 : "), False, False, 0)
            self.mobile = Gtk.Label(self.xml.find("form/data", "mobile"))
            self.mobile.set_alignment(0, 0.5)
            self.mobile.set_selectable(True)
            self.mobile.set_size_request(211, -1)
            hbox.pack_start(self.mobile, False, False, 0)
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "fax.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 팩 스 : "), False, False, 0)
            self.fax = Gtk.Label(self.xml.find("form/data/office", "fax"))
            self.fax.set_alignment(0, 0.5)
            self.fax.set_selectable(True)
            self.fax.set_size_request(220, -1)
            hbox.pack_start(self.fax, False, False, 0)
            self.right.pack_start(hbox, False, False, 1)
            hbox = Gtk.HBox()
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "address.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 주 소 : "), False, False, 0)
            self.address = Gtk.Label(self.xml.find("form/data", "address"))
            self.address.set_alignment(0, 0.5)
            self.address.set_selectable(True)
            hbox.pack_start(self.address, True, True, 0)
            self.right.pack_start(hbox, False, False, 1)
            # vbox.pack_start(Gtk.Label("메 모 :"), True, True, 0)
            self.memo = Gtk.Label()
            self.memo.set_alignment(0, 0)
            self.memo.set_selectable(True)
            self.memo.set_markup(self.xml.find("form/data", "memo"))
            # self.memo.set_size_request(-1, 120)
            self.right.pack_start(self.memo, True, True, 5)

            self.box.pack_start(self.left, False, False, 20)
            self.box.pack_start(self.right, True, True, 20)
            self.box.show_all()
        
            return self.box

        elif self.xml_id == "2006010300":
            # [업체정보] 업체명+사업자번호+대표+업태+종목+주소+전화+팩스+메모
            
            self.box = Gtk.HBox(spacing=5)
            self.box.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0.95, 0.95, 1.0, 1.0))
            self.left = Gtk.VBox(spacing=2)
            self.right = Gtk.VBox(spacing=2)

            self.photo = Gtk.Image()
            self.photo.set_size_request(PREVIEW_SIZE, PREVIEW_SIZE)
            self.photo.set_alignment(0, 0)
            if os.path.isfile(preview):
                _image = Image.open(preview)
                _image.thumbnail((PREVIEW_SIZE, PREVIEW_SIZE), Image.ANTIALIAS)
                image = Image.new("RGBA", (PREVIEW_SIZE, PREVIEW_SIZE))
                image.paste(_image, (int((PREVIEW_SIZE - _image.size[0])/2), 0))
                self.photo.set_from_pixbuf(image_to_pixbuf(image))
            self.left.pack_start(self.photo, False, False, 10)

            hbox = Gtk.HBox()
            self.company = Gtk.Label()
            self.company.set_alignment(0, 0.5)
            self.company.set_selectable(True)
            self.company.set_markup("<span color='darkblue' size='large'><b>%s</b></span>" % self.xml.find("form/data", "company"))
            hbox.pack_start(self.company, False, False, 0)
            self.right.pack_start(hbox, False, False, 10)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("번 호 : "), False, False, 0)
            self.no = Gtk.Label(self.xml.find("form/data", "no"))
            self.no.set_alignment(0, 0)
            self.no.set_selectable(True)
            self.no.set_size_request(220, -1)
            hbox.pack_start(self.no, False, False, 0)
            hbox.pack_start(Gtk.Label("대 표 : "), False, False, 0)
            self.name = Gtk.Label(self.xml.find("form/data", "name"))
            self.name.set_alignment(0, 0)
            self.name.set_selectable(True)
            self.name.set_size_request(220, -1)
            hbox.pack_start(self.name, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("업 태 : "), False, False, 0)
            self.type = Gtk.Label(self.xml.find("form/data", "type"))
            self.type.set_alignment(0, 0)
            self.type.set_selectable(True)
            self.type.set_size_request(220, -1)
            hbox.pack_start(self.type, False, False, 0)
            hbox.pack_start(Gtk.Label("종 목 : "), False, False, 0)
            self.item = Gtk.Label(self.xml.find("form/data", "item"))
            self.item.set_alignment(0, 0)
            self.item.set_selectable(True)
            self.item.set_size_request(220, -1)
            hbox.pack_start(self.item, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("주 소 : "), False, False, 0)
            self.address = Gtk.Label(self.xml.find("form/data", "address"))
            self.address.set_alignment(0, 0)
            self.address.set_selectable(True)
            hbox.pack_start(self.address, True, True, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "phone.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 전 화 : "), False, False, 0)
            self.phone = Gtk.Label(self.xml.find("form/data", "phone"))
            self.phone.set_alignment(0, 0.5)
            self.phone.set_selectable(True)
            self.phone.set_size_request(200, -1)
            hbox.pack_start(self.phone, False, False, 0)
            image = Gtk.Image()
            image.set_from_file(os.path.join(IMAGE_PATH, "form", self.xml_id, "fax.gif"))
            hbox.pack_start(image, False, False, 0)
            hbox.pack_start(Gtk.Label(" 팩 스 : "), False, False, 0)
            self.fax = Gtk.Label(self.xml.find("form/data", "fax"))
            self.fax.set_alignment(0, 0.5)
            self.fax.set_selectable(True)
            self.fax.set_size_request(220, -1)
            hbox.pack_start(self.fax, False, False, 0)
            self.right.pack_start(hbox, False, False, 5)
            # self.right.pack_start(Gtk.Label("메 모 :"), True, True, 0)
            self.memo = Gtk.Label()
            self.memo.set_alignment(0, 0)
            self.memo.set_selectable(True)
            self.memo.set_markup(self.xml.find("form/data", "memo"))
            self.right.pack_start(self.memo, True, True, 7)

            self.box.pack_start(self.left, False, False, 20)
            self.box.pack_start(self.right, True, True, 20)
            self.box.show_all()
        
            return self.box

        elif self.xml_id == "2009051100":
            # [사진메모] 사진+메모
            
            self.box = Gtk.HBox(spacing=5)
            self.box.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0.95, 0.95, 0.95, 1.0))
            self.left = Gtk.VBox(spacing=2)
            self.right = Gtk.VBox(spacing=2)

            self.photo = Gtk.Image()
            self.photo.set_size_request(PREVIEW_SIZE, PREVIEW_SIZE)
            self.photo.set_alignment(0, 0)
            if os.path.isfile(preview):
                _image = Image.open(preview)
                _image.thumbnail((PREVIEW_SIZE, PREVIEW_SIZE), Image.ANTIALIAS)
                image = Image.new("RGBA", (PREVIEW_SIZE, PREVIEW_SIZE))
                image.paste(_image, (int((PREVIEW_SIZE - _image.size[0])/2), 0))
                self.photo.set_from_pixbuf(image_to_pixbuf(image))
            self.left.pack_start(self.photo, False, False, 10)

            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label(" "), False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            self.memo = Gtk.Label()
            self.memo.set_alignment(0, 0)
            self.memo.set_selectable(True)
            self.memo.set_markup(self.xml.find("form/data", "memo"))
            self.right.pack_start(self.memo, True, True, 5)

            self.box.pack_start(self.left, False, False, 20)
            self.box.pack_start(self.right, True, True, 20)
            self.box.show_all()
        
            return self.box

        elif self.xml_id == "2009051200":
            # [책,서적] 커버+제목+저자+번역+장르+등급+출판년도+페이지+출판사+ISBN+프리뷰
            
            self.box = Gtk.HBox(spacing=5)
            self.box.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0.85, 0.85, 1.0, 1.0))
            self.left = Gtk.VBox(spacing=2)
            self.right = Gtk.VBox(spacing=2)

            self.photo = Gtk.Image()
            self.photo.set_size_request(PREVIEW_SIZE, PREVIEW_SIZE)
            self.photo.set_alignment(0, 0)
            if os.path.isfile(preview):
                _image = Image.open(preview)
                _image.thumbnail((PREVIEW_SIZE, PREVIEW_SIZE), Image.ANTIALIAS)
                image = Image.new("RGBA", (PREVIEW_SIZE, PREVIEW_SIZE))
                image.paste(_image, (int((PREVIEW_SIZE - _image.size[0])/2), 0))
                self.photo.set_from_pixbuf(image_to_pixbuf(image))
            self.left.pack_start(self.photo, False, False, 10)

            hbox = Gtk.HBox()
            self.title = Gtk.Label()
            self.title.set_alignment(0, 0.5)
            self.title.set_selectable(True)
            self.title.set_markup("<span color='darkblue' size='large'><b>%s</b></span>" % self.xml.find("form/data", "title"))
            hbox.pack_start(self.title, False, False, 0)
            self.right.pack_start(hbox, False, False, 10)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("저 자 : "), False, False, 0)
            self.author = Gtk.Label(self.xml.find("form/data", "author"))
            self.author.set_alignment(0, 0)
            self.author.set_selectable(True)
            self.author.set_size_request(280, -1)
            hbox.pack_start(self.author, False, False, 0)
            hbox.pack_start(Gtk.Label("번 역 : "), False, False, 0)
            self.translator = Gtk.Label(self.xml.find("form/data", "translator"))
            self.translator.set_alignment(0, 0)
            self.translator.set_selectable(True)
            self.translator.set_size_request(220, -1)
            hbox.pack_start(self.translator, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("장 르 : "), False, False, 0)
            self.genre = Gtk.Label(self.xml.find("form/data", "genre"))
            self.genre.set_alignment(0, 0)
            self.genre.set_selectable(True)
            hbox.pack_start(self.genre, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("등 급 : "), False, False, 0)
            self.rating = Gtk.Label(self.xml.find("form/data", "rating"))
            self.rating.set_alignment(0, 0)
            self.rating.set_selectable(True)
            hbox.pack_start(self.rating, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("출판사 : "), False, False, 0)
            self.publisher = Gtk.Label(self.xml.find("form/data", "publisher"))
            self.publisher.set_alignment(0, 0)
            self.publisher.set_selectable(True)
            self.publisher.set_size_request(270, -1)
            hbox.pack_start(self.publisher, False, False, 0)
            hbox.pack_start(Gtk.Label("출간일 : "), False, False, 0)
            self.year = Gtk.Label(self.xml.find("form/data", "year"))
            self.year.set_alignment(0, 0)
            self.year.set_selectable(True)
            self.year.set_size_request(220, -1)
            hbox.pack_start(self.year, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("분 량 : "), False, False, 0)
            self.pages = Gtk.Label(self.xml.find("form/data", "pages"))
            self.pages.set_alignment(0, 0)
            self.pages.set_selectable(True)
            self.pages.set_size_request(280, -1)
            hbox.pack_start(self.pages, False, False, 0)
            hbox.pack_start(Gtk.Label("ISBN : "), False, False, 0)
            self.isbn = Gtk.Label(self.xml.find("form/data", "isbn"))
            self.isbn.set_alignment(0, 0)
            self.isbn.set_selectable(True)
            self.isbn.set_size_request(220, -1)
            hbox.pack_start(self.isbn, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            self.memo = Gtk.Label()
            self.memo.set_alignment(0, 0)
            self.memo.set_selectable(True)
            self.memo.set_markup(self.xml.find("form/data", "memo"))
            self.right.pack_start(self.memo, True, True, 5)

            self.box.pack_start(self.left, False, False, 20)
            self.box.pack_start(self.right, True, True, 20)
            self.box.show_all()
        
            return self.box

        elif self.xml_id == "2009051300":
            # [영화폴더] 포스터+제목+감독+주연+장르+등급+제작년도+상영시간+배급사
            
            self.box = Gtk.HBox(spacing=5)
            self.box.override_background_color(Gtk.StateType.NORMAL, Gdk.RGBA(0.95, 1.0, 0.95, 1.0))
            self.left = Gtk.VBox(spacing=2)
            self.right = Gtk.VBox(spacing=2)

            self.photo = Gtk.Image()
            self.photo.set_size_request(PREVIEW_SIZE, PREVIEW_SIZE)
            self.photo.set_alignment(0, 0)
            if os.path.isfile(preview):
                _image = Image.open(preview)
                _image.thumbnail((PREVIEW_SIZE, PREVIEW_SIZE), Image.ANTIALIAS)
                image = Image.new("RGBA", (PREVIEW_SIZE, PREVIEW_SIZE))
                image.paste(_image, (int((PREVIEW_SIZE - _image.size[0])/2), 0))
                self.photo.set_from_pixbuf(image_to_pixbuf(image))
            self.left.pack_start(self.photo, False, False, 10)

            hbox = Gtk.HBox()
            self.title = Gtk.Label()
            self.title.set_alignment(0, 0.5)
            self.title.set_selectable(True)
            self.title.set_markup("<span color='darkblue' size='large'><b>%s</b></span>" % self.xml.find("form/data", "title"))
            hbox.pack_start(self.title, False, False, 0)
            self.right.pack_start(hbox, False, False, 10)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("감 독 : "), False, False, 0)
            self.director = Gtk.Label(self.xml.find("form/data", "director"))
            self.director.set_alignment(0, 0)
            self.director.set_selectable(True)
            self.director.set_size_request(250, -1)
            hbox.pack_start(self.director, False, False, 0)
            hbox.pack_start(Gtk.Label("국 가 : "), False, False, 0)
            self.nation = Gtk.Label(self.xml.find("form/data", "nation"))
            self.nation.set_alignment(0, 0)
            self.nation.set_selectable(True)
            self.nation.set_size_request(220, -1)
            hbox.pack_start(self.nation, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("출 연 : "), False, False, 0)
            self.cast = Gtk.Label(self.xml.find("form/data", "cast"))
            self.cast.set_alignment(0, 0)
            self.cast.set_selectable(True)
            self.cast.set_size_request(220, -1)
            hbox.pack_start(self.cast, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("장 르 : "), False, False, 0)
            self.genre = Gtk.Label(self.xml.find("form/data", "genre"))
            self.genre.set_alignment(0, 0)
            self.genre.set_selectable(True)
            self.genre.set_size_request(250, -1)
            hbox.pack_start(self.genre, False, False, 0)
            hbox.pack_start(Gtk.Label("등 급 : "), False, False, 0)
            self.rating = Gtk.Label(self.xml.find("form/data", "rating"))
            self.rating.set_alignment(0, 0)
            self.rating.set_selectable(True)
            self.rating.set_size_request(220, -1)
            hbox.pack_start(self.rating, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("제 작: "), False, False, 0)
            self.producer = Gtk.Label(self.xml.find("form/data", "producer"))
            self.producer.set_alignment(0, 0)
            self.producer.set_selectable(True)
            self.producer.set_size_request(250, -1)
            hbox.pack_start(self.producer, False, False, 0)
            hbox.pack_start(Gtk.Label("배급사: "), False, False, 0)
            self.distributor = Gtk.Label(self.xml.find("form/data", "distributor"))
            self.distributor.set_alignment(0, 0)
            self.distributor.set_selectable(True)
            self.distributor.set_size_request(220, -1)
            hbox.pack_start(self.distributor, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            hbox = Gtk.HBox()
            hbox.pack_start(Gtk.Label("상영시간 : "), False, False, 0)
            self.time = Gtk.Label(self.xml.find("form/data", "time"))
            self.time.set_alignment(0, 0)
            self.time.set_selectable(True)
            self.time.set_size_request(120, -1)
            hbox.pack_start(self.time, False, False, 0)
            hbox.pack_start(Gtk.Label("제작년도 : "), False, False, 0)
            self.year = Gtk.Label(self.xml.find("form/data", "year"))
            self.year.set_alignment(0, 0)
            self.year.set_selectable(True)
            self.year.set_size_request(120, -1)
            hbox.pack_start(self.year, False, False, 0)
            hbox.pack_start(Gtk.Label("개봉일 : "), False, False, 0)
            self.date = Gtk.Label(self.xml.find("form/data", "date"))
            self.date.set_alignment(0, 0)
            self.date.set_selectable(True)
            self.date.set_size_request(120, -1)
            hbox.pack_start(self.date, False, False, 0)
            self.right.pack_start(hbox, False, False, 0)
            # self.right.pack_start(Gtk.Label("메 모 :"), True, True, 0)
            self.memo = Gtk.Label()
            self.memo.set_alignment(0, 0)
            self.memo.set_selectable(True)
            self.memo.set_markup(self.xml.find("form/data", "memo"))
            # self.memo.set_size_request(-1, 120)
            self.right.pack_start(self.memo, True, True, 5)

            self.box.pack_start(self.left, False, False, 20)
            self.box.pack_start(self.right, True, True, 20)
            self.box.show_all()
        
            return self.box

        else:
            # 처리불가
            return None

        
